import 'mocha';
import { TestScheduler } from 'rxjs/testing';
import { of } from 'rxjs';
import { delay, catchError, switchMap } from 'rxjs/operators';
import { throwError } from 'rxjs/internal/observable/throwError';
import * as chai from 'chai';
import * as sinon from 'sinon';
import * as sinonChai from 'sinon-chai';
import connect, { normalClosureMessage } from '.';
chai.use(sinonChai);
var expect = chai.expect;
describe('rxjs-websockets', function () {
    var scheduler;
    var expect$;
    var flush;
    var cold;
    var hot;
    beforeEach(function () {
        scheduler = new TestScheduler(chai.assert.deepEqual);
        expect$ = scheduler.expectObservable.bind(scheduler);
        flush = scheduler.flush.bind(scheduler);
        cold = scheduler.createColdObservable.bind(scheduler);
        hot = scheduler.createHotObservable.bind(scheduler);
    });
    var MockSocket = /** @class */ (function () {
        function MockSocket() {
            this.close = sinon.stub();
        }
        // forwards input as output
        MockSocket.prototype.send = function (data) { this.onmessage({ data: data }); };
        return MockSocket;
    }());
    var connectHelper = function (mockSocket, protocols) {
        return connect('url', { protocols: protocols, makeWebSocket: function () { return mockSocket; } });
    };
    it('connects to websocket lazily and retrieves data', function () {
        var mockSocket = new MockSocket();
        var socket = connectHelper(mockSocket);
        var input = hot('abcde|');
        expect$(of(null).pipe(
        // delay subscription to websocket by 10ms, TODO: find some way to
        // verify the connection is attempted lazily
        delay(10, scheduler), switchMap(function () {
            return socket.pipe(switchMap(function (factory) {
                // ensure factory is called when socket is open
                expect(scheduler.now()).to.equal(20);
                return factory(input);
            }));
        }))).toBe('--cde');
        // websocket opens at 20ms
        scheduler.schedule(function () {
            // if one of the expectations raises an error this won't be defined
            if (mockSocket.onopen)
                mockSocket.onopen();
        }, 20);
        flush();
    });
    it('closes websocket on unsubscribe', function () {
        var mockSocket = new MockSocket();
        var socket = connectHelper(mockSocket);
        scheduler.schedule(function () { return mockSocket.onopen(); }, 10);
        expect$(socket.pipe(switchMap(function (factory) { return factory(cold('a|')); })), '--!').toBe('-a');
        flush();
        expect(mockSocket.close).to.have.been.calledOnce;
    });
    describe('raises Error', function () {
        var runTest = function (reason, code, expectedReason) {
            var mockSocket = new MockSocket();
            var socket = connectHelper(mockSocket);
            scheduler.schedule(function () { return mockSocket.onopen(); }, 10);
            scheduler.schedule(function () { return mockSocket.onclose({ reason: reason, code: code }); }, 30);
            expect$(socket.pipe(switchMap(function (factory) { return factory(cold('a')); }), 
            // rethrow error as string... can't get expectation to match the error
            catchError(function (error) { return throwError(error.message); }))).toBe('-a-#', undefined, expectedReason);
            flush();
        };
        it('with message equal to reason on unclean websocket close', function () {
            var reason = 'Freakish closure';
            runTest(reason, 9000, reason);
        });
        it('with normalClosureMessage when socket was closed normally', function () {
            runTest('whatever', 1000, normalClosureMessage);
        });
    });
});
//# sourceMappingURL=index.spec.js.map